@@ -67,17 +67,17 @@ static void TranslateToDos(doscp_t *toDos, const char *in, char *out,
 			continue;
 		}
 
-		if (iswcntrl(*s)) {
+		if (iswcntrl((wint_t)*s)) {
 			/* "control" characters */
 			*mangled |= 3;
 			buffer[t_idx] = '_';
-		} else if (iswlower(*s)) {
+		} else if (iswlower((wint_t)*s)) {
 			buffer[t_idx] = ch_towupper(*s);
 			if(*Case == UPPER && !mtools_no_vfat)
 				*mangled |= 1;
 			else
 				*Case = LOWER;
-		} else if (iswupper(*s)) {
+		} else if (iswupper((wint_t)*s)) {
 			buffer[t_idx] = *s;
 			if(*Case == LOWER && !mtools_no_vfat)
 				*mangled |= 1;
@@ -110,7 +110,7 @@ void dos_name(doscp_t *toDos, const char *name, int verbose UNUSEDP,
 		name = &name[2];
 
 	/* zap the leading path */
-	name = _basename(name);
+	name = mt_basename(name);
 	if ((s = strrchr(name, '\\')))
 		name = s + 1;
 
@@ -190,3 +190,27 @@ wchar_t *unix_name(doscp_t *dosCp,
 	return ret;
 }
 
+int isSpecial(const char *name)
+{
+	if(name[0] == '\0')
+		return 1;
+	if(!strcmp(name,"."))
+		return 1;
+	if(!strcmp(name,".."))
+		return 1;
+	return 0;
+}
+
+#ifdef HAVE_WCHAR_H
+int isSpecialW(const wchar_t *name)
+{
+	if(name[0] == '\0')
+		return 1;
+	if(!wcscmp(name,L"."))
+		return 1;
+	if(!wcscmp(name,L".."))
+		return 1;
+	return 0;
+}
+#endif
+
