@@ -16,12 +16,12 @@
  */
 
 #include "sysincludes.h"
-#include "msdos.h"
+#include "mtools.h"
 
-const char *mversion="4.0.38";
+const char *mversion="4.0.44";
 
 /* Multiple releases on same day should be marked with (b), (cd), (d) after
  * date string below */
-const char *mdate = "March 3rd, 2022";
+const char *mdate = "June 2nd, 2024";
 
-const char *mformat_banner = "MTOO4038";
+const char *mformat_banner = "MTOO4044";
