@@ -1,4 +1,21 @@
-/*
+/*  Copyright 1996   Grant R. Guenther,  based on work of Itai Nahshon
+ *   http://www.torque.net/ziptool.html
+ *  Copyright 1997-1999,2001,2002,2005,2007,2009 Alain Knaff.
+ *  This file is part of mtools.
+ *
+ *  Mtools is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  Mtools is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
+ *
  * scsi.c
  * Iomega Zip/Jaz drive tool
  * change protection mode and eject disk
@@ -19,6 +36,8 @@
 #include "mtools.h"
 #include "scsi.h"
 
+#ifdef HAVE_SCSI
+
 #if defined OS_hpux
 #include <sys/scsi.h>
 #endif
@@ -37,12 +56,8 @@
 #endif
 
 #ifdef OS_linux
-#define SCSI_IOCTL_SEND_COMMAND 1
-struct scsi_ioctl_command {
-    int  inlen;
-    int  outlen;
-    char cmd[5008];
-};
+#include <scsi/scsi.h>
+#include <scsi/sg.h>
 #endif
 
 #ifdef _SCO_DS
@@ -53,7 +68,11 @@ struct scsi_ioctl_command {
 #include <camlib.h>
 #endif
 
-int scsi_max_length(void)
+#if defined(OS_netbsd) || defined(OS_netbsdelf)
+#include <sys/scsiio.h>
+#endif
+
+unsigned int scsi_max_length(void)
 {
 #ifdef OS_linux
 	return 8;
@@ -62,7 +81,8 @@ int scsi_max_length(void)
 #endif
 }
 
-int scsi_open(const char *name, int flag, int mode, void **extra_data)
+int scsi_open(const char *name, int flag UNUSEDP, int mode UNUSEDP,
+	      void **extra_data UNUSEDP)
 {
 #if (defined(OS_freebsd)) && (__FreeBSD__ >= 2)
     struct cam_device *cam_dev;
@@ -81,12 +101,12 @@ int scsi_open(const char *name, int flag, int mode, void **extra_data)
 #endif
 }
 
-int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
-	     void *data, size_t len, void *extra_data)
+int scsi_cmd(int fd, unsigned char cdb[6], uint8_t cmdlen, scsi_io_mode_t mode,
+	     void *data, uint32_t len, void *extra_data UNUSEDP)
 {
 #if defined OS_hpux
 	struct sctl_io sctl_io;
-	
+
 	memset(&sctl_io, 0, sizeof sctl_io);   /* clear reserved fields */
 	memcpy(sctl_io.cdb, cdb, cmdlen);      /* copy command */
 	sctl_io.cdb_length = cmdlen;           /* command length */
@@ -98,7 +118,7 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 			sctl_io.data_length = len;
 			sctl_io.data = data;
 			break;
-		case SCSI_IO_WRITE: 
+		case SCSI_IO_WRITE:
 			sctl_io.flags = 0;
 			sctl_io.data_length = data ? len : 0;
 			sctl_io.data = len ? data : 0;
@@ -111,7 +131,7 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 	}
 
 	return sctl_io.cdb_status;
-	
+
 #elif defined OS_sunos || defined OS_solaris
 	struct uscsi_cmd uscsi_cmd;
 	memset(&uscsi_cmd, 0, sizeof uscsi_cmd);
@@ -120,7 +140,7 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 #ifdef OS_solaris
 	uscsi_cmd.uscsi_timeout = 20;  /* msec? */
 #endif /* solaris */
-	
+
 	uscsi_cmd.uscsi_buflen = (u_int)len;
 	uscsi_cmd.uscsi_bufaddr = data;
 
@@ -140,45 +160,41 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 
 	if(uscsi_cmd.uscsi_status) {
 		errno = 0;
-		fprintf(stderr,"scsi status=%x\n",  
+		fprintf(stderr,"scsi status=%x\n",
 			(unsigned short)uscsi_cmd.uscsi_status);
 		return -1;
 	}
-	
+
 	return 0;
-	
+
 #elif defined OS_linux
-	struct scsi_ioctl_command my_scsi_cmd;
+	struct sg_io_hdr my_scsi_cmd;
 
+	/*
+	** Init the command
+	*/
+	memset(&my_scsi_cmd,0,sizeof(my_scsi_cmd));
+	my_scsi_cmd.interface_id    = 'S';
+	my_scsi_cmd.dxfer_direction = (mode == SCSI_IO_READ)?(SG_DXFER_FROM_DEV):(SG_DXFER_TO_DEV);
+	my_scsi_cmd.cmd_len         = cmdlen;
+	my_scsi_cmd.mx_sb_len       = 0;
+	my_scsi_cmd.dxfer_len       = len;
+	my_scsi_cmd.dxferp          = data;
+	my_scsi_cmd.cmdp            = cdb;
+	my_scsi_cmd.timeout         = ~0u; /* where is MAX_UINT defined??? */
 
-	memcpy(my_scsi_cmd.cmd, cdb, cmdlen);        /* copy command */
+#ifdef DEBUG
+	printf("CMD(%d): %02x%02x%02x%02x%02x%02x %sdevice\n",cmdlen,cdb[0],cdb[1],cdb[2],cdb[3],cdb[4],cdb[5],
+		(mode==SCSI_IO_READ)?("<-"):("->"));
+	printf("DATA   : len = %d\n",len);
+#endif
 
-	switch (mode) {
-		case SCSI_IO_READ:
-			my_scsi_cmd.inlen = 0;
-			my_scsi_cmd.outlen = len;
-			break;
-		case SCSI_IO_WRITE:
-			my_scsi_cmd.inlen = len;
-			my_scsi_cmd.outlen = 0;
-			memcpy(my_scsi_cmd.cmd + cmdlen,data,len);
-			break;
-	}
-	
-	if (ioctl(fd, SCSI_IOCTL_SEND_COMMAND, &my_scsi_cmd) < 0) {
+	if (ioctl(fd, SG_IO,&my_scsi_cmd) < 0) {
 		perror("scsi_io");
 		return -1;
 	}
-	
-	switch (mode) {
-		case SCSI_IO_READ:
-			memcpy(data, &my_scsi_cmd.cmd[0], len);
-			break;
-		case SCSI_IO_WRITE:
-			break;
-    }
 
-	return 0;  /* where to get scsi status? */
+	return my_scsi_cmd.status & STATUS_MASK;
 
 #elif (defined _SCO_DS) && (defined SCSIUSERCMD)
 	struct scsicmd my_scsi_cmd;
@@ -213,7 +229,7 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 	case SCSI_IO_WRITE:
 	  my_scsi_cmd.ds_flags = DSRQ_WRITE|DSRQ_SENSE;
 	  break;
-        } 
+        }
 	my_scsi_cmd.ds_time = 10000;
 	my_scsi_cmd.ds_link = 0;
 	my_scsi_cmd.ds_synch =0;
@@ -225,11 +241,11 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
 
         if(my_scsi_cmd.ds_status) {
                 errno = 0;
-                fprintf(stderr,"scsi status=%x\n",  
+                fprintf(stderr,"scsi status=%x\n",
                         (unsigned short)my_scsi_cmd.ds_status);
                 return -1;
         }
-        
+
         return 0;
 #elif (defined OS_freebsd) && (__FreeBSD__ >= 2)
 #define MSG_SIMPLE_Q_TAG 0x20 /* O/O */
@@ -264,14 +280,48 @@ int scsi_cmd(int fd, unsigned char *cdb, int cmdlen, scsi_io_mode_t mode,
                     96,
                     cmdlen,
                     5000);
-                    
+
       if (cam_send_ccb(cam_dev, ccb) < 0 ||
 	  (ccb->ccb_h.status & CAM_STATUS_MASK) != CAM_REQ_CMP) {
 	  return -1;
       }
       return 0;
+#elif defined(OS_netbsd) || defined(OS_netbsdelf)
+ 	struct scsireq sc;
+
+	memset(&sc, 0, sizeof(sc));
+	memcpy(sc.cmd, cdb, cmdlen);
+	sc.cmdlen = cmdlen;
+	sc.databuf = data;
+	sc.datalen = len;
+	sc.senselen = 0;
+	sc.timeout = 10000;
+	switch (mode) {
+	case SCSI_IO_READ:
+	  sc.flags = SCCMD_READ;
+	  break;
+	case SCSI_IO_WRITE:
+	  sc.flags = SCCMD_WRITE;
+	  break;
+	}
+
+	if (ioctl(fd, SCIOCCOMMAND, &sc) == -1) {
+                perror("SCIOCCOMMAND ioctl");
+                return -1;
+	}
+
+	if (sc.retsts) {
+                errno = EIO;
+                fprintf(stderr, "SCSI command failed, retsts %d\n",
+sc.retsts);
+                return -1;
+	}
+
+        return 0;
 #else
       fprintf(stderr, "scsi_io not implemented\n");
       return -1;
 #endif
 }
+#endif
+
